"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserRepository = void 0;
const tslib_1 = require("tslib");
const bcrypt = tslib_1.__importStar(require("bcrypt"));
const berrors_1 = require("berrors");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const authentication_1 = require("@bleco/authentication");
const core_2 = require("@loopx/core");
const models_1 = require("../models");
const types_1 = require("../types");
const phone_1 = require("../utils/phone");
const otp_repository_1 = require("./otp.repository");
const user_credentials_repository_1 = require("./user-credentials.repository");
const saltRounds = 10;
let UserRepository = exports.UserRepository = class UserRepository extends core_2.DefaultUserUpdatableCrudRepository {
    constructor(dataSource, getUserCredsRepository, getOtpRepository, getCurrentUser, tenantRepositoryGetter, userTenantRepositoryGetter, logger, entityClass = models_1.User) {
        super(entityClass, dataSource, getCurrentUser);
        this.getOtpRepository = getOtpRepository;
        this.tenantRepositoryGetter = tenantRepositoryGetter;
        this.userTenantRepositoryGetter = userTenantRepositoryGetter;
        this.logger = logger;
        this.userTenants = this.createHasManyRepositoryFactoryFor('userTenants', userTenantRepositoryGetter);
        this.registerInclusionResolver('userTenants', this.userTenants.inclusionResolver);
        this.tenant = this.createBelongsToAccessorFor('defaultTenant', tenantRepositoryGetter);
        this.registerInclusionResolver('defaultTenant', this.tenant.inclusionResolver);
        this.credentials = this.createHasOneRepositoryFactoryFor('credentials', getUserCredsRepository);
        this.registerInclusionResolver('credentials', this.credentials.inclusionResolver);
    }
    async create(entity, options) {
        var _a, _b;
        options = options !== null && options !== void 0 ? options : {};
        if (!entity.username && !entity.email && !entity.phone) {
            throw new rest_1.HttpErrors.BadRequest('Username or email or phone is required');
        }
        // Here "User" just for type
        const or = [];
        if (entity.username) {
            or.push({ username: entity.username });
        }
        if (entity.email) {
            or.push({ email: entity.email });
        }
        if (entity.phone) {
            or.push({ phone: (0, phone_1.normalizePhoneNumber)(entity.phone) });
        }
        const userExists = await super.findOne({
            where: { or },
        });
        if (userExists) {
            throw new rest_1.HttpErrors.BadRequest('User already exists');
        }
        const user = await super.create(entity, options);
        try {
            let creds;
            const authProvider = (_a = options.authProvider) !== null && _a !== void 0 ? _a : (options.authId ? 'keycloak' : 'internal');
            const authId = options.authId;
            switch (authProvider) {
                case 'internal': {
                    const password = (_b = options.password) !== null && _b !== void 0 ? _b : process.env.USER_TEMP_PASSWORD;
                    if (!password) {
                        throw new rest_1.HttpErrors.UnprocessableEntity('Password is required');
                    }
                    const hashedPassword = await bcrypt.hash(password, saltRounds);
                    creds = new models_1.UserCredentials({
                        authProvider: 'internal',
                        password: hashedPassword,
                    });
                    break;
                }
                default: {
                    creds = new models_1.UserCredentials({
                        authProvider,
                        authId,
                    });
                    break;
                }
            }
            await this.credentials(user.id).create(creds, options);
        }
        catch (err) {
            await super.deleteByIdHard(user.id);
            throw new rest_1.HttpErrors.UnprocessableEntity('Error while hashing password');
        }
        return user;
    }
    async createWithoutCreds(entity, options) {
        return super.create(entity, options);
    }
    async verifyPassword(username, password) {
        const user = await super.findOne({
            where: { username: username.toLowerCase() },
        });
        const creds = user && (await this.credentials(user.id).get());
        if (!user || user.deleted) {
            throw new core_2.AuthErrors.UserDoesNotExist();
        }
        else if (!(creds === null || creds === void 0 ? void 0 : creds.password) ||
            creds.authProvider !== core_2.AuthProvider.INTERNAL ||
            !(await bcrypt.compare(password, creds.password))) {
            this.logger.error('User creds not found in DB or is invalid');
            throw new authentication_1.AuthenticationErrors.InvalidCredentials();
        }
        else {
            return user;
        }
    }
    async updatePassword(username, password, newPassword) {
        const user = await super.findOne({ where: { username } });
        const creds = user && (await this.credentials(user.id).get());
        if (!user || user.deleted || !creds || !creds.password) {
            throw new core_2.AuthErrors.UserDoesNotExist();
        }
        else if (creds.authProvider !== core_2.AuthProvider.INTERNAL) {
            throw new core_2.AuthErrors.PasswordCannotBeChanged();
        }
        else if (!(await bcrypt.compare(password, creds.password))) {
            throw new authentication_1.AuthenticationErrors.WrongPassword();
        }
        else if (await bcrypt.compare(newPassword, creds.password)) {
            throw new berrors_1.BErrors.Unauthorized('Password cannot be same as previous password!');
        }
        else {
            // Do nothing
        }
        await this.credentials(user.id).patch({
            password: await bcrypt.hash(newPassword, saltRounds),
        });
        return user;
    }
    async changePassword(username, newPassword, oldPassword) {
        const user = await super.findOne({ where: { username } });
        const creds = user && (await this.credentials(user.id).get());
        if (oldPassword) {
            // This method considers old password as OTP
            const otp = await (await this.getOtpRepository()).get(username);
            if (!otp || otp.otp !== oldPassword) {
                throw new authentication_1.AuthenticationErrors.WrongPassword();
            }
        }
        if ((creds === null || creds === void 0 ? void 0 : creds.authProvider) !== core_2.AuthProvider.INTERNAL) {
            throw new core_2.AuthErrors.PasswordCannotBeChanged();
        }
        if (!user || user.deleted || !creds || !creds.password) {
            throw new core_2.AuthErrors.UserDoesNotExist();
        }
        else if (await bcrypt.compare(newPassword, creds.password)) {
            throw new berrors_1.BErrors.Unauthorized('Password cannot be same as previous password!');
        }
        else {
            // DO nothing
        }
        await this.credentials(user.id).patch({
            password: await bcrypt.hash(newPassword, saltRounds),
        });
        return user;
    }
    async updateLastLogin(userId) {
        await super.updateById(userId, {
            lastLogin: Date.now(),
        }, {
            currentUser: { id: userId },
        });
    }
    async firstTimeUser(userId) {
        const user = await super.findOne({
            where: {
                id: userId,
            },
        });
        if (!user) {
            throw new core_2.AuthErrors.UserDoesNotExist();
        }
        const userTenant = await (await this.userTenantRepositoryGetter()).findOne({
            where: {
                userId,
                tenantId: user.defaultTenantId,
                status: {
                    inq: [core_2.UserStatus.REGISTERED, core_2.UserStatus.PASSWORD_CHANGE_NEEDED],
                },
            },
        });
        return !!userTenant;
    }
};
exports.UserRepository = UserRepository = tslib_1.__decorate([
    tslib_1.__param(0, (0, core_1.inject)(`datasources.${types_1.AuthDbSourceName}`)),
    tslib_1.__param(1, repository_1.repository.getter(user_credentials_repository_1.UserCredentialsRepository)),
    tslib_1.__param(2, repository_1.repository.getter(otp_repository_1.OtpRepository)),
    tslib_1.__param(3, core_1.inject.getter(authentication_1.AuthenticationBindings.CURRENT_USER, { optional: true })),
    tslib_1.__param(4, repository_1.repository.getter('TenantRepository')),
    tslib_1.__param(5, repository_1.repository.getter('UserTenantRepository')),
    tslib_1.__param(6, (0, core_1.inject)(core_2.LOGGER.LOGGER_INJECT)),
    tslib_1.__metadata("design:paramtypes", [repository_1.juggler.DataSource, Function, Function, Function, Function, Function, Object, Object])
], UserRepository);
//# sourceMappingURL=user.repository.js.map